/**
 * The test class SortedBufferTest.
 *
 * @author  Uno Holmer
 * @version 2010-09-09
 */
public class SortedBufferTest extends junit.framework.TestCase
{
    private SortedBuffer emptyBuffer;
    private SortedBuffer buffer1full;
    private SortedBuffer buffer2equal;
    private SortedBuffer buffer2unequal;
    private SortedBuffer buffer3unequal;

   /**
     * Default constructor for test class SortedBufferTest
     */
    public SortedBufferTest()
    {
    }

    /**
     * Sets up the test fixture.
     *
     * Called before every test case method.
     */
    protected void setUp()
    {
        emptyBuffer = new SortedBuffer(1);
        buffer1full = new SortedBuffer(1);
        buffer1full.insert(123);
        buffer2equal = new SortedBuffer(3);
        buffer2equal.insert(2);
        buffer2equal.insert(2);
        buffer2unequal = new SortedBuffer(3);
        buffer2unequal.insert(1);
        buffer2unequal.insert(3);
        buffer3unequal = new SortedBuffer(3);
        buffer3unequal.insert(1);
        buffer3unequal.insert(2);
        buffer3unequal.insert(3);
    }

    /**
     * Tears down the test fixture.
     *
     * Called after every test case method.
     */
    protected void tearDown()
    {
    }
        
    // Simple cases
    public void testInsertEmpty()
    {
        assertEquals(true,emptyBuffer.insert(123));
        assertEquals(1,emptyBuffer.size());
    }

    public void testInsertFull()
    {
        assertEquals(false,buffer1full.insert(456));
    }

    
    // Order cases
    
    public void testInsertBeforeTwoEqual()
    {
        assertEquals(true,buffer2equal.isOrdered());
        int n = buffer2equal.size();
        buffer2equal.insert(1);
        assertEquals(n+1,buffer2equal.size());
        assertEquals(true,buffer2equal.isOrdered());
    }
    
    public void testInsertAfterTwoEqual()
    {
        assertEquals(true,buffer2equal.isOrdered());
        int n = buffer2equal.size();
        buffer2equal.insert(3);
        assertEquals(n+1,buffer2equal.size());
        assertEquals(true,buffer2equal.isOrdered());
    }
    
    public void testInsertBefore2Unequal()
    {
        assertEquals(true, buffer2unequal.isOrdered());
        int n = buffer2unequal.size();
        buffer2unequal.insert(0);
        assertEquals(n+1, buffer2unequal.size());
        assertEquals(true, buffer2unequal.isOrdered());
    }
    
    public void testInsertBetween2Unequal()
    {
        assertEquals(true, buffer2unequal.isOrdered());
        int n = buffer2unequal.size();
        buffer2unequal.insert(2);
        assertEquals(n+1, buffer2unequal.size());
        assertEquals(true, buffer2unequal.isOrdered());
    }
    
    public void testInsertAfter2Unequal()
    {
        assertEquals(true, buffer2unequal.isOrdered());
        int n = buffer2equal.size();
        buffer2unequal.insert(4);
        assertEquals(n+1, buffer2unequal.size());
        assertEquals(true, buffer2unequal.isOrdered());
    }
    
    public void testRemoveEmptyBuffer()
    {
        assertEquals(false, emptyBuffer.remove(-1));
        assertEquals(false, emptyBuffer.remove(0));
        assertEquals(false, emptyBuffer.remove(1));
    }
    
    public void testRemoveLastElement()
    {
        int n = buffer1full.size();
        assertEquals(true, buffer1full.remove(0));
        assertEquals(n-1, buffer1full.size());
    }
    
    public void testRemoveFirstOfThree()
    {
        int n = buffer3unequal.size();   
        assertEquals(true, buffer3unequal.isOrdered());
        assertEquals(true, buffer3unequal.remove(0));
        assertEquals(n-1, buffer2unequal.size());
        assertEquals(true, buffer3unequal.isOrdered());
    }
    
    public void testRemoveMiddleOfThree()
    {
        int n = buffer3unequal.size();  
        assertEquals(true, buffer3unequal.isOrdered());
        assertEquals(true, buffer3unequal.remove(1));
        assertEquals(n-1, buffer2unequal.size());
        assertEquals(true, buffer3unequal.isOrdered());
    }

    public void testRemoveLastOfThree()
    {
        int n = buffer3unequal.size();  
        assertEquals(true, buffer3unequal.isOrdered());
        assertEquals(true, buffer3unequal.remove(2));
        assertEquals(n-1, buffer2unequal.size());
        assertEquals(true, buffer3unequal.isOrdered());
    }
}

































